use sled::{Db, IVec};
use thiserror::Error;

#[derive(Clone)]
pub struct Data {
    db: Db,
}

#[derive(Error, Debug)]
pub enum Error {
    #[error("sled error: {0}")]
    Sled(#[from] sled::Error),
}

impl Data {
    pub fn try_new() -> Result<Self, Error> {
        Ok(Self {
            db: sled::open("data/lyrs")?,
        })
    }

    pub fn get<K: AsRef<[u8]>, V: From<IVec>>(
        &self,
        tree_name: &str,
        key: K,
    ) -> Result<Option<V>, Error> {
        Ok(self
            .db
            .open_tree(tree_name)?
            .get(key.as_ref())?
            .map(V::from))
    }

    pub fn insert<K: AsRef<[u8]>, V: Into<IVec>>(
        &self,
        tree_name: &str,
        key: K,
        value: V,
    ) -> Result<Option<IVec>, Error> {
        Ok(self.db.open_tree(tree_name)?.insert(key, value.into())?)
    }
}
