use crate::prelude::*;
use redact::Secret;
use serde::Deserialize;
use thiserror::Error;

pub const USER_TREE: &str = "user";

#[derive(Deserialize, Debug)]
pub struct User {
    pub username: String,
    password_digest: Secret<String>,
}

#[derive(Error, Debug)]
pub enum Error {
    #[error("username exists: {0}")]
    UsernameExists(Box<String>),

    #[error("username not found: {0}")]
    UsernameNotFound(Box<String>),
}

impl User {
    pub const fn tree() -> &'static str {
        USER_TREE
    }

    pub fn try_new(username: &str, password: &str) -> Result<Self, argon2::password_hash::Error> {
        Ok(Self {
            username: username.to_owned(),
            password_digest: Secret::new(crate::auth::password_digest(password)?.to_string()),
        })
    }

    pub fn verify(&self, password: &str) -> Result<(), argon2::password_hash::Error> {
        crate::auth::verified_password(password, self.password_digest.expose_secret())
    }
}
